/**
 * Copyright (C) 2010-2011 J.W.Marsden
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package jsonij.parser;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 * Internationised Parser Exception.
 * 
 * @author openecho
 * @version 1.0.0
 */
public abstract class ParserException extends BaseParserException {
    /**
     * Exception Key
     */
    protected String key;
    /**
     * Exception Locale
     */
    protected Locale locale;

    /**
     * Basic Constructor.
     *
     * @param key Exception key
     * @param args Additional Arguments for Exception
     */
    public ParserException(String key, Object... args) {
        this(key, -1, -1, null, args);
    }

    /**
     * Constructor Including Line Number and Position Number of Exception
     *
     * @param key Exception Key
     * @param line Exception Line
     * @param position Exception Position
     * @param args Additional Arguments for Exception
     */
    public ParserException(String key, int line, int position, Object... args) {
        this(key, line, position, null, args);
    }

    /**
     * Constructor Including Line Number, Position Number and Locale of Exception.
     *
     * @param key Exception Key
     * @param line Exception Line
     * @param position Exception Position
     * @param locale Valid Locale for the exception
     * @param args Additional Arguments for Exception
     */
    public ParserException(String key, int line, int position, Locale locale, Object... args) {
        this.line = line;
        this.position = position;
        this.key = key;

        /**
         * TODO:
         * Fix this. Dont force it to be English!
         */
        this.locale = ((locale == null) ? Locale.ENGLISH : locale);
        if (this.locale != null) {
            String messageFormat = ResourceBundle.getBundle(getBundleName()).getString(this.key);
            this.message = String.format(messageFormat, args);
        } else {
            this.message = String.format("Undefined Exception %s %s", key, locale);
        }
    }

    public abstract String getBundleName();
}
